//
// Created by steven on 11/14/2024.
//

#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>

#include "../libs/go_p2fa.h"
#include "images.h"
#include "../libs/e-Paper/EPD_2in13_V4.h"
#include "../libs/GUI/GUI_Paint.h"
#include "../libs/GUI/GUI_BMPfile.h"

void Handler(int sigNum) {
    printf("Caught signal %d\n", sigNum);
    EPD_2in13_V4_Init();
    EPD_2in13_V4_Clear();

    Debug("Goto Sleep...\r\n");
    EPD_2in13_V4_Sleep();

    DEV_Delay_ms(2000);//important, at least 2s
    Debug("close 5V, Module enters 0 power consumption ...\r\n");

    DEV_Module_Exit();
    exit(0);
}

int draw() {
    Paint_NewImage(BlackImage, EPD_2in13_V4_WIDTH, EPD_2in13_V4_HEIGHT, 90, WHITE);
    Debug("Drawing\r\n");
    //1.Select Image
    EPD_2in13_V4_Init();
    Paint_SelectImage(BlackImage);
    Paint_Clear(WHITE);

    // 2.Drawing on the image
    Paint_DrawPoint(5, 10, BLACK, DOT_PIXEL_1X1, DOT_STYLE_DFT);
    Paint_DrawPoint(5, 25, BLACK, DOT_PIXEL_2X2, DOT_STYLE_DFT);
    Paint_DrawPoint(5, 40, BLACK, DOT_PIXEL_3X3, DOT_STYLE_DFT);
    Paint_DrawPoint(5, 55, BLACK, DOT_PIXEL_4X4, DOT_STYLE_DFT);

    Paint_DrawLine(20, 10, 70, 60, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
    Paint_DrawLine(70, 10, 20, 60, BLACK, DOT_PIXEL_1X1, LINE_STYLE_SOLID);
    Paint_DrawRectangle(20, 10, 70, 60, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
    Paint_DrawRectangle(85, 10, 135, 60, BLACK, DOT_PIXEL_1X1, DRAW_FILL_FULL);

    Paint_DrawLine(45, 15, 45, 55, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
    Paint_DrawLine(25, 35, 70, 35, BLACK, DOT_PIXEL_1X1, LINE_STYLE_DOTTED);
    Paint_DrawCircle(45, 35, 20, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
    Paint_DrawCircle(110, 35, 20, WHITE, DOT_PIXEL_1X1, DRAW_FILL_FULL);

    Paint_DrawString(140, 15, "waveshare", &Font16, BLACK, WHITE);
    Paint_DrawNum(140, 40, 123456789, &Font16, BLACK, WHITE);

    EPD_2in13_V4_Display_Base(BlackImage);
    DEV_Delay_ms(3000);

    Paint_NewImage(BlackImage, EPD_2in13_V4_WIDTH, EPD_2in13_V4_HEIGHT, 90, WHITE);
    Debug("Partial refresh\r\n");
    Paint_SelectImage(BlackImage);

    PAINT_TIME sPaint_time;
    sPaint_time.Hour = 12;
    sPaint_time.Min = 34;
    sPaint_time.Sec = 56;
    UBYTE num = 10;
    for (;;) {
        sPaint_time.Sec = sPaint_time.Sec + 1;
        if (sPaint_time.Sec == 60) {
            sPaint_time.Min = sPaint_time.Min + 1;
            sPaint_time.Sec = 0;
            if (sPaint_time.Min == 60) {
                sPaint_time.Hour =  sPaint_time.Hour + 1;
                sPaint_time.Min = 0;
                if (sPaint_time.Hour == 24) {
                    sPaint_time.Hour = 0;
                    sPaint_time.Min = 0;
                    sPaint_time.Sec = 0;
                }
            }
        }
        Paint_ClearWindows(150, 80, 150 + Font20.Width * 7, 80 + Font20.Height, WHITE);
        Paint_DrawTime(150, 80, &sPaint_time, &Font20, WHITE, BLACK);

        num = num - 1;
        if(num == 0) {
            break;
        }
        EPD_2in13_V4_Display_Partial(BlackImage);
        DEV_Delay_ms(500);//Analog clock 1s

        DEV_Delay_ms(5000);

        Debug("Clear...\r\n");
        EPD_2in13_V4_Init();
        EPD_2in13_V4_Clear();

        Debug("Goto Sleep...\r\n");
        EPD_2in13_V4_Sleep();
        free(BlackImage);
        BlackImage = NULL;

        DEV_Delay_ms(2000);//important, at least 2s
        // close 5V
        Debug("close 5V, Module enters 0 power consumption ...\r\n");

        DEV_Module_Exit();
        return 0;
}

int drawn() {
    if (DEV_Module_Init() != 0) {
        return -1;
    }
    EPD_2in13_V4_Init();
    EPD_2in13_V4_Clear();

    UBYTE *BlackImage;
    UWORD ImageSize = ((EPD_2in13_V4_WIDTH % 8 == 0) ? (EPD_2in13_V4_WIDTH / 8) : (EPD_2in13_V4_WIDTH / 8 + 1)) * EPD_2in13_V4_HEIGHT;
    if ((BlackImage = (UBYTE *) malloc(ImageSize)) == NULL) {
        Debug("Failed to apply for black memory...\n");
        return -1;
    }
    Paint_NewImage(BlackImage, EPD_2in13_V4_WIDTH, EPD_2in13_V4_HEIGHT, 90, WHITE);
    Paint_Clear(WHITE);

    EPD_2in13_V4_Init();

    Paint_SelectImage(BlackImage);
    Paint_DrawString(10, 40, "Hello, World!", &Font16, WHITE, BLACK);

    const char *confName = "default";
    GoString confStr = {confName, strlen(confName)};
    const char *code = getCode(confStr);
    Debug("Code: %s\n", code);
    Paint_DrawString(5, 5, code, &Font24, WHITE, BLACK);

    Paint_DrawRectangle(10, 112, 240, 92, BLACK, DOT_PIXEL_1X1, DRAW_FILL_EMPTY);
    EPD_2in13_V4_Display_Base(BlackImage);

    Paint_NewImage(BlackImage, EPD_2in13_V4_WIDTH, EPD_2in13_V4_HEIGHT, 90, WHITE);
    Paint_SelectImage(BlackImage);

    clock_t startTime;
    clock_t endTime;
    double elapsedTime;
    for (int j = 0; j < 3; ++j) {
        for (int i = getTimeRemainingMS(30); i > 500; i = getTimeRemainingMS(30)) {
            startTime = clock();
            Debug("Time Left: %dms\n", i);
            int progWidth = (int)(((double)i / (double)(30 * 1000)) * 240);
            Debug("Progress bar width: %d\n", progWidth);
            Paint_DrawRectangle(10, 112, 240 - progWidth, 102, BLACK, DOT_PIXEL_1X1, DRAW_FILL_FULL);
            Paint_DrawRectangle(10, 102, 240 - progWidth, 92, WHITE, DOT_PIXEL_1X1, DRAW_FILL_FULL);
            EPD_2in13_V4_Display(BlackImage);
            endTime = clock();
            elapsedTime = (double)(endTime - startTime) / CLOCKS_PER_SEC;
            if (elapsedTime < 200) {
                DEV_Delay_ms(200 - (elapsedTime * 1000));
            }
        }
    }

    DEV_Delay_ms(5000);

    Debug("Clear...\r\n");
    EPD_2in13_V4_Init();
    EPD_2in13_V4_Clear();

    Debug("Goto Sleep...\r\n");
    EPD_2in13_V4_Sleep();
    free(BlackImage);
    BlackImage = NULL;

    DEV_Delay_ms(2000);//important, at least 2s
    // close 5V
    Debug("close 5V, Module enters 0 power consumption ...\r\n");

    DEV_Module_Exit();
    return 0;
}

int main() {
    signal(SIGINT, Handler);

    loadConfigs();

    draw();

    return 0;
}